#  
# - SKADS Simulator -
# 
# Based on the classical AIPS task uvcon
#
# 
# Hans - Rainer Kloeckner
# hrk@astro.ox.ac.uk
#
# date: 23.02.2009
#
# version 1.0
#
# Software requirements:
# - python 
# - ParselTongue  (http://www.jive.nl/dokuwiki/doku.php/parseltongue:parseltongue)
# - Obit     FIRST LOOK INTO THE ParselTongue INSTALLATION PAGE  (http://www.cv.nrao.edu/~bcotton/Obit.html)
#
# - and optional AIPS
#
# The following files you need:
#
# - SKADS_SIMULATOR.py
# - AIPSLite.py, AIPSLiteTask.py (-- Stephen Bourke's super stuff -- these files are in the tar file 
#                                           or can be found via http://www.jive.nl/~bourke/aipslite/)
#
# THE IMAGE REQUIREMENTS: dimension is based 2 and positions must be defined in the image centre 
#
# 
# YOU NEED TO SET THIS
#
i_do_have_aips = 1
docleanup      = 1
# ---------
#
import os
cwd = os.getcwd()
#

import AIPSLite
#

if i_do_have_aips == 1:
    try:
        #
        path = os.environ['AIPS_ROOT']    
    except KeyError:
        import sys
        print  '\n\nAIPS has not been defined, please source your LOGIN.CSH\n'
        print 'You may want to use one of these files:\n'
        os.system('locate LOGIN.CSH')
        sys.exit(-1)
    else:
        path = os.environ['AIPS_ROOT']    
    # IF YOU HAVE AN AIPS INSTALATION ON YOUR COMPUTER USE and sourced LOGIN.CSH file 
    #
    AIPSLite.init_environ(path=os.environ['AIPS_ROOT'], version=os.environ['AIPS_VERSION'][len(os.environ['AIPS_VERSION'])-7:])

else:
    #
    # IF YOU DON'T HAVE A AIPS VERSION INSTALLED 
    # 
    print  '\n\nThe default AIPS version ',AIPSLite.default_version,' will be downloaded.\n\n'
    #
    AIPSLite.init_environ(path=cwd, version=AIPSLite.default_version)
    #
    # Get all necessary Task's from NRAO
    #
    AIPSLite.get_task('slice')
    AIPSLite.get_task('sl2pl')
    AIPSLite.get_task('fittp')
    AIPSLite.get_task('fitld')
    AIPSLite.get_task('uvcon')
    AIPSLite.get_task('uvplt')
    AIPSLite.get_task('imagr')
    AIPSLite.get_task('subim')
    AIPSLite.get_task('mcube')
    AIPSLite.get_task('blwup')
    AIPSLite.get_task('imean')
    AIPSLite.get_task('lwpla')
    #
    AIPSLite.get_template()
    AIPSLite.get_popsdat()
    AIPSLite.get_libs()
    #
    os.environ['PLOTTER'] = '/tmp'
#
# make the AIPS directories 
#
AIPSLite.make_da00(cwd+'/da00')
AIPSLite.make_disk(cwd+'/aipsdisk')
#
# ---------------------------------
#
import AIPS
import AIPSLiteTask
from AIPSLiteTask import AIPSTask
from AIPSTask import AIPSList
from AIPSData import AIPSUVData, AIPSImage
from Wizardry.AIPSData import AIPSUVData as WizAIPSUVData
#
#
def datatype(data):
    """
    return the datatype its kind of a hack so do not change !!
    """
    catalog_in = `AIPSCat(data[3])`
    catalog = catalog_in.split()
    #
    source, newdata = [],[]
    for i in range(len(catalog)):
        if catalog[i].count('name') == 1 or catalog[i].count('NAME'):
            newdata.append(i)
    if len(newdata) > 0:
        newdata.append(len(catalog))

    for i in range(len(newdata)-1):
        source.append(str(catalog[newdata[i]:newdata[i+1]]).replace('"','')\
                          .replace(':','').replace(',','').replace('[','').replace(']','')\
                          .replace('{','').replace('}','').split())
    datatype = 'None'
    for i in range(len(source)):
        issour = [-1,-1,-1]
        dtype = 'None'
        datatype = 'None'
        for j in range(len(source[i])-1):
            if eval(source[i][j]) == 'name' and str(eval(source[i][j+1])) == data[0]:
                issour[0] = 1
            if eval(source[i][j]) == 'klass' and str(eval(source[i][j+1])) == data[1]:
                issour[1] = 1
            if eval(source[i][j]) == 'seq' and int(eval(source[i][j+1])) == data[2]:
                issour[2] = 1
            if eval(source[i][j]) == 'type':
                dtype = eval(source[i][j+1])
            if issour[0] == 1 and issour[1] == 1 and issour[2] == 1:
                datatype = dtype
                return(datatype)
    return(datatype)

# --------

def ps2jpg(filename,dpi=90,outname='None'):
    """
    use gs to convert ps files into jpeg
    """
    if outname == 'None':
        filename1 = filename.replace('ps','')
        filename1 = filename1.replace('PS','')
        ps2jpg = 'gs -q -dBATCH -dNOPROMPT -dNOPAUSE -sDEVICE=jpeg -r'+str(dpi)+\
            ' -sOutputFile='+str(filename1)+'jpeg '+filename
        outf = str(filename1)+'jpeg '
    else:
        ps2jpg = 'gs -q -dBATCH -dNOPROMPT -dNOPAUSE -sDEVICE=jpeg -r'+str(dpi)+\
            ' -sOutputFile='+str(outname)+'.jpeg '+filename
        outf = str(outname)+'.jpeg '
    #
    os.system(ps2jpg)
    return(outf)

def checkaipsimdata(data):
    """
    aborts the program if the file is not present in the AIPS disk
    data vector in PT convention to access the file 
    [NAME, CLASS, DISK, SEQ]
    """
    if (datatype(data) == 'UV'):
        aipsdata = AIPSUVData(data[0],data[1],data[2],data[3]) 
    if(datatype == 'MA'):
        aipsdata = AIPSImage(data[0],data[1],data[2],data[3])
    if aipsdata.exists() == False:
        return(-1)
    else:
        return(1)


def getntable(data,inext='AN'):
        """
        returns the number of speficied table 
        """
        if (datatype(data) == 'UV'):
            from Wizardry.AIPSData import AIPSUVData as AIPSDataTAB            
        else:
            from Wizardry.AIPSData import AIPSImage as AIPSDataTAB

        data   = AIPSDataTAB(data[0],data[1],data[2],data[3])
        tables = data.tables
        ntable = -1
        for i in range(len(tables)):
                if tables[i][1] == 'AIPS '+inext:
                        ntable = tables[i][0]
        return(ntable)

def getimheader(data,gethead='FREQ'):
        """
        returns the value, referenc, increment, dimention of the 
        image of the header mostly used to get
        e.g. 'RA---SIN', 'DEC--SIN','STOKES', 'FREQ',
        """
        from AIPSData import AIPSImage
        image    = AIPSImage(data[0],data[1],data[2],data[3])
        imaxis   = image.header['ctype']
        imval    = -1
        imref    = -1
        imincr   = -1
        imdim    = -1

        for i in range(len(imaxis)):
                if imaxis[i] == gethead:
                        imval  = image.header['crval'][i]
                        imref  = image.header['crpix'][i]
                        imincr = image.header['cdelt'][i]
                        imdim  = image.header['naxis'][i]
                        
        return(imval,imref,imincr,imdim)


def putimheader(data,puthead='FREQ',putkeyword='crval',keyvalue=-9999):
        """
        returns the value, referenc, increment, dimention of the 
        image of the header mostly used to get
        e.g. 'RA---SIN', 'DEC--SIN','STOKES', 'FREQ',
        """
	from Wizardry.AIPSData import AIPSImage as WizAIPSImage
	#image = WizAIPSImage(subim.outname,subim.outclass,int(subim.outseq),int(subim.outdisk))


        image    = WizAIPSImage(data[0],data[1],data[2],data[3])
        imaxis   = image.header['ctype']
        imval    = -1
        imref    = -1
        imincr   = -1
        imdim    = -1

        for i in range(len(imaxis)):
                if imaxis[i] == puthead:
			if keyvalue != -9999:
			    image.header[putkeyword][i] = keyvalue
			    image.header.update()
		        imval  = image.header['crval'][i]
                        imref  = image.header['crpix'][i]
                        imincr = image.header['cdelt'][i]
                        imdim  = image.header['naxis'][i]
                        #
        return(imval,imref,imincr,imdim)




def pl2ps(data,psname,dpl=-1):
    """
    produce ps file from a pl file and delete it afterwards  
    """
    import os

    # take care about the sting lengh in AIPS    
    if len(psname) > 40:
        psname = psname[:40]
    #
    if (datatype(data) == 'UV'):
	from Wizardry.AIPSData import AIPSUVData as WizAIPSUVData
        pldata = WizAIPSUVData(data[0],data[1],data[2],data[3])
    else:
	from Wizardry.AIPSData import AIPSImage as WizAIPSImage
        pldata = WizAIPSImage(data[0],data[1],data[2],data[3])
    #
    plver = getntable(data,'PL')
    #
    if plver > 0:
        lwpla          = AIPSTask('lwpla')
        lwpla.indata   = pldata
        lwpla.plver    = 1
        lwpla.invers   = plver
        lwpla.dparm[5] = 1
        lwpla.lpen     = 1
        lwpla.docolor  = -1
        lwpla.outfile  = 'PWD:'+psname+'.ps'
        #lwpla.outfile  = 'PWD:'+'XCY'+'.ps'
        lwpla.msgkill  = -2
        lwpla.go()

        # erase all PL tables
        pldata.zap_table('PL', dpl)

        # rename it (AIPS can cause errors if name it to long)
        #renameit = '/bin/mv XCY.ps '+psname+'.ps'
        #os.system(renameit)

        return(psname+'.ps')
    else:
        return('')


def makebeamplots(data,psname='BEAM'):
    """
    produce a cross cut through the synthesised beam
    """

    rimval,rimref,rimincr,rimdim = getimheader(data,'RA---SIN')
    dimval,dimref,dimincr,dimdim = getimheader(data,'DEC--SIN')
    #
    blc1 = [int(rimdim/2.0),1] # vertical cut
    trc1 = [int(dimdim/2.0),dimdim]
    blc2 = [1,int(rimdim/2.0)] # horizontal cut
    trc2 = [dimdim,int(dimdim/2.0)]
    #
    image    = AIPSImage(data[0],data[1],data[2],data[3])
    #
    slice          = AIPSTask('slice')
    slice.inname   = data[0]
    slice.inclass  = data[1]
    slice.inseq    = data[2]
    slice.indisk   = data[3]
    slice.blc[1:]  = blc1
    slice.trc[1:]  = trc1
    slice.msgkill  = -2
    slice.go()
    #
    sl2pl          = AIPSTask('sl2pl')
    sl2pl.inname   = data[0]
    sl2pl.inclass  = data[1]
    sl2pl.inseq    = data[2]
    sl2pl.indisk   = data[3]
    sl2pl.msgkill  = -2
    sl2pl.go()
    #
    image.zap_table('SL', -1)
    #
    pl2ps(data,psname)
    #
    slice          = AIPSTask('slice')
    slice.inname   = data[0]
    slice.inclass  = data[1]
    slice.inseq    = data[2]
    slice.indisk   = data[3]
    slice.blc[1:]  = trc2
    slice.trc[1:]  = blc2
    slice.msgkill  = -2
    slice.go()
    #
    sl2pl          = AIPSTask('sl2pl')
    sl2pl.inname   = data[0]
    sl2pl.inclass  = data[1]
    sl2pl.inseq    = data[2]
    sl2pl.indisk   = data[3]
    sl2pl.msgkill  = -2
    sl2pl.go()
    #
    image.zap_table('SL', -1)
    #
    pl2ps(data,psname)
    #
    return(psname)

def calpixvalim(data,axis='Freq',pixel=0):
    """
    Calculates the correct value of the pixel
    according to email exchange with Eric Geisen 
    freq = freq0 + (ch - refch) * increment
    """
    headinfo = getimheader(data,axis)
    if headinfo[0] != -1:
        if pixel == 0:
            value = headinfo[0] + (headinfo[3] - headinfo[1]) * headinfo[2]
        else:
            value = headinfo[0] + (pixel - headinfo[1]) * headinfo[2]
        return(value)
    else:
        return(-1)

def checkfi(chfi='None'):
    """
    check if file is present input string 
    will give back 
    value 1 of if present
         -1 not present
    # 06.04.2006
    """
    try:
       # Check path/file is present
       checkfile=1
       os.stat(str(chfi))
    except os.error:
       checkfile=-1
       pass
    return(checkfile)


def save2disk(data,fname,file2image,fnamext,doclean=-1,noutput=-2):
        """
        save the aips file into the current directory
        the filename: will be build via fname+file2image+fnamext
        doclean = 1 will delete file from AIPS DISK
        """
        from AIPSTask import AIPSTask
        from os import getcwd
        #
        file2save = fname+file2image+fnamext
        if len(file2save) > 43:
            file2save = file2image+fnamext
        cwd = getcwd()
        #
        # Save final tables to disk
        #
        fittp              = AIPSTask('fittp')
        fittp.inname       = data[0]
        fittp.inclass      = data[1]
        fittp.indisk       = int(data[2])
        fittp.inseq        = int(data[3])
        fittp.dataout      = 'PWD:'+file2save
        fittp.msgkill      = noutput
        fittp.go()
        #

 
	if doclean > 0:
	    image    = AIPSImage(data[0],data[1],data[2],data[3])
	    image.zap()
        #
        return()



def getimstat(data,percentage=5,imsize='None',noutput=-2):
        """
        Determine and return the std, average, min, max, xy of max of an image
. 
        """
        pct = percentage/100.
        if imsize == 'None':
                image = AIPSImage(data[0],data[1],int(data[2]),int(data[3]))
                imsize = image.header['naxis'][0]
                if 1 > imsize:
                        print '\CAUTION: Dimension of Image axis less < 1\n'
                        sys.exit(-1)
        #
        imean         = AIPSTask('imean')
        imean.inname  = data[0]
        imean.inclass = data[1]
        imean.indisk  = int(data[2])
        imean.inseq   = int(data[3])
        imean.blc[1:] = [int(pct*imsize),int(pct*imsize)]
        imean.trc[1:] = [int((1-pct)*imsize),int((1-pct)*imsize)]
        imean.pixavg  = 0 
        imean.pixstd  = 0
        imean.docat   = -1
        imean.msgkill = noutput
        imean.go()
        # check the explain file of imstat
        imean.go()

        fluxstd = imean.pixstd
        fluxavg = imean.pixavg

        fluxmax = image.header.datamax
        fluxmin = image.header.datamin

        return(fluxstd,fluxavg,fluxmin,fluxmax)


def uvselvis(data,stokes,ant=0,base=0,chan=1,IF=1,timer=[0,99999],ret=5):
	"""
	return vectors of selected visibilities
	time,bsl,uvdis,fg,amp,phas,re,im
	ret = 3 returns time,bsl,uvdis,fg,amp,phas
	ret = 5 returns time,bsl,uvdis,fg,amp,phas,re,im
	Caution with the means; it will calculate values from the entire vector
	ret > 5 returns time,bsl,uvdis,fg,[phas,<phase>,sigma],
	        [amp,<amp>,sigma],[re,<re>,sigma],[im,<im>,sigma]
	"""
	#
	from Wizardry.AIPSData import AIPSUVData as AIPSDataVIS
	from math import atan2,sqrt,pi
	#from Numeric.MLab import mean,std
	from MLab import mean,std
	#
	visdata = AIPSDataVIS(data[0],data[1],data[2],data[3])
	phas,amp,re,im,time,uvdis = [],[],[],[],[],[]
	bsl = []
	fg  = []
	#
	IF   = int(IF - 1)
	chan = int(chan - 1)
	#
	if stokes =='RR':
		st = 0
	elif stokes =='LL':
		st = 1
	elif stokes =='RL':
		st = 2
	elif stokes =='LR':
		st = 3
	else:
		print '\nStokes need to be specified\n'
		sys.exit(-1)

	for visibility in visdata:
		if visibility.time > timer[0] and visibility.time < timer[1]:
			if ant == 0 and base == 0:
				sre   = visibility.visibility[IF,chan,st,0]
				sim   = visibility.visibility[IF,chan,st,1]
				#
				u = visibility.uvw[0]
				v = visibility.uvw[1]
				w = visibility.uvw[2]
				uvdis.append(sqrt(u**2+v**2+w**2))
				#
				time.append(visibility.time)
				phas.append((360./(2*pi)*atan2(sim, sre)))
				amp.append(sqrt(sre**2+sim**2))
				re.append(visibility.visibility[IF,chan,st,0])
				im.append(visibility.visibility[IF,chan,st,1])
				bsl.append(visibility.baseline)
				fg.append(-1)
			elif ant == 0 and base != 0:
				if visibility.baseline[1] == base or visibility.baseline[0] == base: 
					sre   = visibility.visibility[IF,chan,st,0]
					sim   = visibility.visibility[IF,chan,st,1]
					#
                                        u = visibility.uvw[0]
                                        v = visibility.uvw[1]
                                        w = visibility.uvw[2]
                                        uvdis.append(sqrt(u**2+v**2+w**2))
					#
					time.append(visibility.time)
					phas.append((360./(2*pi)*atan2(sim, sre)))
					amp.append(sqrt(sre**2+sim**2))
					re.append(visibility.visibility[IF,chan,st,0])
					im.append(visibility.visibility[IF,chan,st,1])
					bsl.append(visibility.baseline)	    
					fg.append(-1)
			elif ant != 0 and base == 0:
				if visibility.baseline[1] == ant or visibility.baseline[0] == ant: 
					sre   = visibility.visibility[IF,chan,st,0]
					sim   = visibility.visibility[IF,chan,st,1]
					#
                                        u = visibility.uvw[0]
                                        v = visibility.uvw[1]
                                        w = visibility.uvw[2]
                                        uvdis.append(sqrt(u**2+v**2+w**2))
                                        #
					time.append(visibility.time)
					phas.append((360./(2*pi)*atan2(sim, sre)))
					amp.append(sqrt(sre**2+sim**2))
					re.append(visibility.visibility[IF,chan,st,0])
					im.append(visibility.visibility[IF,chan,st,1])
					bsl.append(visibility.baseline)	   
					fg.append(-1)
			else:
				if visibility.baseline == [ant, base]:		    
					sre   = visibility.visibility[IF,chan,st,0]
					sim   = visibility.visibility[IF,chan,st,1]
					#
                                        u = visibility.uvw[0]
                                        v = visibility.uvw[1]
                                        w = visibility.uvw[2]
                                        uvdis.append(sqrt(u**2+v**2+w**2))
					#
					time.append(visibility.time)
#					phas.append((360./(2*pi)*atan2(sre, sim)))
					phas.append((360./(2*pi)*atan2(sim, sre)))
					amp.append(sqrt(sre**2+sim**2))
					re.append(visibility.visibility[IF,chan,st,0])
					im.append(visibility.visibility[IF,chan,st,1])
					bsl.append(visibility.baseline)
					fg.append(-1)
	if ret > 5:
	        if 3 > len(amp)  :
		        print '\nBaseline ',ant,'-',base,' does not exist.\n'
			return([-1])
		else:
			return(time,bsl,uvdis,fg,[amp,mean(amp), std(amp)],\
			       [phas,mean(phas),std(phas)], [re, mean(re), std(re)],\
			       [im, mean(im), std(im)])
	elif ret == 5:
		return(time,bsl,uvdis,fg,amp,phas,re,im)
	else:
		return(time,bsl,uvdis,fg,amp,phas)




def simchanmerge(data,antennafile,pmha=6,intpvismin=30,domerge=1,tmdata='None'):
    """
    will simulate one channel
    from an input model
    pmha       = +- hour angle
    intpvismin = integration per visibility [min]
    """
    #
    dochangenoise = -1
    if 2 * pmha > 24: 
        from math import sqrt
        dochangenoise = sqrt(24/(2.0*pmha)) 
        pmha = 12
    #
    nchannels = 1
    intvisi   = intpvismin*60     # s integration per visibility
    anteleva  = 15                # deg

    freqdim   = getimheader(data,gethead='FREQ')
    nchannels = freqdim[3]
    dfreq     = freqdim[2]
    obsfreqc  = freqdim[1]
    obsfreq   = freqdim[0]
    chanwidth = dfreq/1E6

    decimage   = getimheader(data,gethead='DEC--SIN')
    # determine the center of the image
    if int(decimage[1]) != int(decimage[3]/2.):
        decimcentre =  decimage[0] + (decimage[3]/2. - decimage[1] + 1) * decimage[2]
    else:
        decimcentre = decimage[0]

    # Here produce the UVfits file
    uvcon            = AIPSTask('UVCON')
    uvcon.infile     = 'PWD:'+str(antennafile)
    uvcon.outdisk    = outdata
    uvcon.outname    = 'CHA'
    uvcon.outclass   = 'SIM'
    uvcon.outseq     = int(workchannel)
    uvcon.in2name    = data[0]
    uvcon.in2class   = data[1]
    uvcon.in2disk    = int(data[2])
    uvcon.in2seq     = int(data[3])
    uvcon.cmodel     = 'IMAG'
    uvcon.aparm[1]   = (obsfreq/1E9)          # Frequency of lowest frequency channel, GHz
    uvcon.aparm[2]   = 0                      # only 1 or 2 should be set
    uvcon.aparm[3]   = decimcentre            # source declination
    uvcon.aparm[4]   = (-1*pmha)              # Min hour angle, hours, related to the array centre
    uvcon.aparm[5]   = pmha                   # Max hour angle, hours
    uvcon.aparm[6]   = anteleva               # Min antenna elevation, deg
    uvcon.aparm[7]   = intvisi                # Integration time per visibility point, sec
    uvcon.aparm[8]   = chanwidth              # Bandwidth(Increment) of frequency channel, MHz
    uvcon.aparm[9]   = nchannels      
    if dochangenoise != -1:
        uvcon.bparm[1] = dochangenoise 
    uvcon.do3dimag   = 1
    uvcon.msgkill    = noutput

    uvcon.go()
    uvdata = [uvcon.outname,uvcon.outclass,int(uvcon.outdisk),int(uvcon.outseq)]

    if domerge == 1 and tmdata != 'None':
	dbcon          = AIPSTask('dbcon')
	dbcon.indisk   = int(uvdata[2])
	dbcon.in2disk  = int(tmdata[2])
	dbcon.outdisk  = int(uvdata[2])
	dbcon.reweight[1:] = [1,1]
	dbcon.inname   = uvcon.outname
	dbcon.in2name  = tmdata[0]
	dbcon.outname  = 'CH'+str(workchannel)
	dbcon.outclass = 'CHMRG'
	dbcon.inseq    = int(uvdata[3])
	dbcon.in2seq   = tmdata[3]
	dbcon.doarray  = 1
	dbcon.fqtol    = -1
	dbcon.msgkill  = noutput
	dbcon.go()
	#
	#
	buvdata = AIPSUVData(uvcon.outname,uvcon.outclass,int(1),int(workchannel))
	buvdata.zap()
	#
	auvdata = AIPSUVData(tmdata[0],tmdata[1],tmdata[2],tmdata[3])
	auvdata.zap()
	#
	uvdata  = [dbcon.outname,dbcon.outclass,int(dbcon.outdisk),int(1)]


    return(uvdata)


def mkimage(uvdat,data,cellsi=[0,0],imagesize=-1,cccomp=200,wrobust=0):
    """
    will simulate one channel
    from an input model
    pmha       = +- hour angle
    intpvismin = integration per visibility [min]
    """

    uvdata = AIPSUVData(uvdat[0],uvdat[1],uvdat[2],uvdat[3])
    #
    uvplt = 1
    if uvplt == 1:
	uvplot           = AIPSTask('uvplt')
	uvplot.indata    = uvdata
	uvplot.docalib   = -1
	uvplot.doband    = -1
	#uvplot.bparm[1:] = [9,10]
        #uvplot.bparm[1:] = [14,15]
	uvplot.bparm[1:] = [6,7,2]
	uvplot.dotv      = -1
	uvplot.go()
	ps2jpg(pl2ps(uvdat,'SM_UVPLT_CHA'+str(1)))
	os.system('/bin/rm SM_UVPLT_CHA'+str(workchannel)+'.ps')
    #

    if cellsi == [0,0]:
	from math import pi
	vis      = uvselvis(uvdat,'RR')
	uvdismax = max(vis[2])
	light_c  = 299792458.0        # m s-1
	wl       = light_c/obsfreq
	fwhm     = 1.02*wl/uvdismax*360.0*60.*60./(2.0*pi)  # synthezied beam to 1st null
	cellsi  = [fwhm/3,fwhm/3]



    # input image info
    raimage    = getimheader(data,gethead='RA---SIN')
    decimage   = getimheader(data,gethead='DEC--SIN')

    # determine the center of the image
    if int(decimage[1]) != int(decimage[3]/2.):
        decimcentre =  decimage[0] + (decimage[3]/2. - decimage[1] + 1) * decimage[2]
    else:
        decimcentre = decimage[0]

    if imagesize == -1:
	imsi    = [decimage[3],decimage[3]]
    else:
	imsi    = [imagesize,imagesize]

    fstokes ='I'
    #
    #
    # IMAGING
    #
    imagr              = AIPSTask('imagr')
    imagr.indata       = uvdata
    imagr.stokes       = fstokes
    imagr.docalib      = -1
    imagr.doband       = -1
    imagr.flagver      = -1
    imagr.nfield       = 1
    imagr.nchav        = nchannels
    imagr.cellsi[1:]   = cellsi
    imagr.imsi[1:]     = imsi
    imagr.robust       = wrobust
    imagr.niter        = cccomp
    imagr.outdisk      = outdata
    imagr.msgkill      = noutput
    imagr.go()
    #
    # Plot Beam slices
    # vertical than horizontal
    makebeamplots([uvdat[0],'IBM001',int(1),1],'SM_BEAM_CUTS')
    #
    uvdata.zap()
    imdata = AIPSImage(uvdat[0],'IBM001',int(1),int(1))
    imdata.zap()
    imdata = AIPSImage(data[0],data[1],data[2],data[3])
    imdata.clrstat()
    imdata.zap()
    


    # determine the center of the image in RA
    if int(raimage[1]) != int(raimage[3]/2.):
#        raimcentre =  raimage[0] + (raimage[3]/2. - raimage[1] + 1) * raimage[2]
        raimcentre =  raimage[0] + (raimage[3]/2. - raimage[1]) * raimage[2]
    else:
        raimcentre = raimage[0]

    # change the coordinate centre to the input image centre 
    putimheader([uvdat[0],'ICL001',int(1),int(1)],'RA---SIN','crval',raimcentre)
    #
    newraimage    = getimheader([uvdat[0],'ICL001',int(1),int(1)],gethead='RA---SIN')
    putimheader([uvdat[0],'ICL001',int(1),int(1)],'RA---SIN','crpix',int(newraimage[3]/2.))

    return([uvdat[0],'ICL001',int(1),1],[cellsi])

def cropimage(data):
    """
    croped an image and return the name of the new image
    """
    import math
    #
    imdata = AIPSImage(data[0],data[1],data[2],data[3])
    #
    raimdim  = getimheader(data,'RA---SIN')[3]
    decimdim = getimheader(data,'DEC--SIN')[3]
    #
    if math.log(raimdim)/math.log(2) - int(math.log(raimdim)/math.log(2)) > 0:
        i = 0
        while raimdim > 2**i:
            cblc = [1,1]
            ctrc = [(2**i),(2**i)]
            i = i + 1

        print '\nImage will be croped to ',cblc,ctrc
        print 'Image original dimension ',raimdim,'\n\n'

        subim             = AIPSTask('subim')
        subim.indata      = imdata
        subim.outname     = data[0]
        subim.outclass    = 'CROP'
        subim.outseq      = 1
        subim.outdisk     = int(indata)
        subim.blc[1:]     = cblc
        subim.trc[1:]     = ctrc
        subim.msgkill     = noutput
        subim.go()
        #
        imdata.zap()
        #
        data = [subim.outname,subim.outclass,int(subim.outseq),int(subim.outdisk)]
    #
    ra = getimheader(data,'RA---SIN')
    dec = getimheader(data,'DEC--SIN')
    raimdim  = ra[3]
    decimdim = dec[3]
    #
    ranref    = raimdim/2.
    decnref   = decimdim/2.
    #
    if abs(ranref - ra[1]) > 5:
        print '\nCAUTION -'
        print 'The image coordinates must be defined in the image centre'
        racepix = calpixvalim(data,axis='RA---SIN',pixel=ranref)
        print 'e.g. for RA ',racepix,' at pixel ',ranref,'will be redefined\n\n' 
        putimheader(data,'RA---SIN','crval',calpixvalim(data,axis='RA---SIN',pixel=ranref))
        putimheader(data,'RA---SIN','crpix',ranref)
    if abs(decnref - dec[1]) > 5:
        print '\nCAUTION -'
        print 'The image coordinates must be defined in the image centre'
        deccepix  = calpixvalim(data,axis='DEC--SIN',pixel=ranref)
        print 'e.g. for DEC ',deccepix,' at pixel ',decnref,'will be redefined\n\n' 
        putimheader(data,'DEC--SIN','crval',calpixvalim(data,axis='DEC--SIN',pixel=decnref))
        putimheader(data,'DEC--SIN','crpix',decnref)
    #
    return(data)

def simchan(data,antennafile,pmha=6,intpvismin=30,cellsi=[0,0],imagesize=-1,cccomp=200,wrobust=0):
    """
    will simulate one channel
    from an input model
    pmha       = +- hour angle
    intpvismin = integration per visibility [min]
    """
    #
    global workchannel
    #
    nchannels = 1
    intvisi   = intpvismin*60     # s integration per visibility
    anteleva  = 15                # deg

    dochangenoise = -1
    if 2 * pmha > 24: 
        from math import sqrt
        dochangenoise = sqrt(24/(2.0*pmha)) 
        pmha = 12

    freqdim   = getimheader(data,gethead='FREQ')
    nchannels = freqdim[3]
    dfreq     = freqdim[2]
    obsfreqc  = freqdim[1]
    obsfreq   = freqdim[0]
    chanwidth = dfreq/1E6

    # input image info
    raimage    = getimheader(data,gethead='RA---SIN')
    decimage   = getimheader(data,gethead='DEC--SIN')

    # determine the center of the image
    if int(decimage[1]) != int(decimage[3]/2.):
        decimcentre =  decimage[0] + (decimage[3]/2. - decimage[1] + 1) * decimage[2]
    else:
        decimcentre = decimage[0]

    # Here produce the UVfits file
    uvcon            = AIPSTask('UVCON')
    uvcon.infile     = 'PWD:'+str(antennafile)
    uvcon.outdisk    = outdata
    uvcon.outname    = 'CHA'
    uvcon.outclass   = 'SIM'
    uvcon.in2name    = data[0]
    uvcon.in2class   = data[1]
    uvcon.in2disk    = int(data[2])
    uvcon.in2seq     = int(data[3])
    uvcon.cmodel     = 'IMAG'
    uvcon.aparm[1]   = (obsfreq/1E9)          # Frequency of lowest frequency channel, GHz
    uvcon.aparm[2]   = 0                      # only 1 or 2 should be set
    uvcon.aparm[3]   = decimcentre            # source declination
    uvcon.aparm[4]   = (-1*pmha)              # Min hour angle, hours, related to the array centre
    uvcon.aparm[5]   = pmha                   # Max hour angle, hours
    uvcon.aparm[6]   = anteleva               # Min antenna elevation, deg
    uvcon.aparm[7]   = intvisi                # Integration time per visibility point, sec
    uvcon.aparm[8]   = chanwidth              # Bandwidth(Increment) of frequency channel, MHz
    uvcon.aparm[9]   = nchannels      

    if dochangenoise != -1:
        uvcon.bparm[1] = dochangenoise 

    uvcon.do3dimag   = 1
    uvcon.msgkill    = noutput

    uvcon.go()
    #
    uvdata = AIPSUVData(uvcon.outname,uvcon.outclass,int(1),1)
    #
    uvplt = 1
    if uvplt == 1 and workchannel == 1:
	uvplot           = AIPSTask('uvplt')
	uvplot.indata    = uvdata
	uvplot.docalib   = -1
	uvplot.doband    = -1
	#uvplot.bparm[1:] = [9,10]
        #uvplot.bparm[1:] = [14,15]
	uvplot.bparm[1:] = [6,7,2]
	uvplot.dotv      = -1
	uvplot.go()
	ps2jpg(pl2ps([uvcon.outname,uvcon.outclass,int(1),1],'SM_UVPLT_CHA'+str(workchannel)))
	os.system('/bin/rm SM_UVPLT_CHA'+str(workchannel)+'.ps')
    #

    if cellsi == [0,0]:
	from math import pi
	vis      = uvselvis([uvcon.outname,uvcon.outclass,int(1),1],'RR')
	uvdismax = max(vis[2])
	light_c  = 299792458.0        # m s-1
	wl       = light_c/obsfreq
	fwhm     = 1.02*wl/uvdismax*360.0*60.*60./(2.0*pi)  # synthezied beam to 1st null
	cellsi  = [fwhm/3,fwhm/3]


    if imagesize == -1:
	imsi    = [decimage[3],decimage[3]]
    else:
	imsi    = [imagesize,imagesize]

    fstokes ='I'
    #
    #
    # IMAGING
    #
    imagr              = AIPSTask('imagr')
    imagr.indata       = uvdata
    imagr.stokes       = fstokes
    imagr.docalib      = -1
    imagr.doband       = -1
    imagr.flagver      = -1
    imagr.nfield       = 1
    imagr.nchav        = nchannels
    imagr.cellsi[1:]   = cellsi
    imagr.imsi[1:]     = imsi
    imagr.robust       = wrobust
    imagr.niter        = cccomp
    imagr.outdisk      = outdata
    imagr.msgkill      = noutput
    imagr.go()
    #
    # Plot Beam slices
    # vertical than horizontal
    makebeamplots([uvcon.outname,'IBM001',int(1),1],'SM_BEAM_CUTS')
    # Clean up
    uvdata.zap()
    imdata = AIPSImage(uvcon.outname,'IBM001',int(1),1)
    imdata.zap()
    imdata = AIPSImage(data[0],data[1],data[2],data[3])
    imdata.clrstat()
    imdata.zap()
    
    # determine the center of the image in RA
    if int(raimage[1]) != int(raimage[3]/2.):
#        raimcentre =  raimage[0] + (raimage[3]/2. - raimage[1] + 1) * raimage[2]
        raimcentre =  raimage[0] + (raimage[3]/2. - raimage[1]) * raimage[2]
    else:
        raimcentre = raimage[0]

    # change the coordinate centre to the input image centre 
    putimheader([uvcon.outname,'ICL001',int(1),int(workchannel)],'RA---SIN','crval',raimcentre)
    #
    newraimage    = getimheader([uvcon.outname,'ICL001',int(1),int(workchannel)],gethead='RA---SIN')
    putimheader([uvcon.outname,'ICL001',int(1),int(workchannel)],'RA---SIN','crpix',int(newraimage[3]/2.))

    return([uvcon.outname,'ICL001',int(1),1],[cellsi])

def chanextract(data,cha):
    """
    """

    freqinp = getimheader(data,'FREQ')

    if cha > freqinp[3]:
	print 'Channel does not exist'
	sys.exit(-1)

    imdata = AIPSImage(data[0],data[1],data[2],data[3])

    freqref = freqinp[0] + (freqinp[1] - 1) * freqinp[2] 
    chafreq = freqref + cha * freqinp[2]

    subim             = AIPSTask('subim')
    subim.indata      = imdata
    subim.outname     = 'CA'
    subim.outclass    = 'XIM'
    subim.outseq      = cha
    subim.outdisk     = int(indata)
    subim.blc[1:]     = [0,0,cha]
    subim.trc[1:]     = [0,0,cha]
    subim.msgkill     = noutput
    subim.go()

    chaim = ['CA','XIM',1,int(cha)]

    putimheader(chaim,'FREQ','crval',chafreq)
    putimheader(chaim,'FREQ','crpix',1)

    return(chaim)

def chanmerge(data,cha,cleanup = 1):
    """
    produce a cube
    """
    imdata = AIPSImage(data[0],data[1],data[2],data[3])
    #
    mcube             = AIPSTask('mcube')
    mcube.indata      = imdata
    mcube.in2seq      = int(cha+data[3]-1)
    mcube.in3seq      = 1
    mcube.npoints     = cha
    mcube.outname     = 'CUBE'
    mcube.outclass    = 'XIM'
    mcube.outseq      = 1
    mcube.outdisk     = int(indata)
    mcube.msgkill     = noutput
    mcube.go()
    #
    if cleanup == 1:
	for i in range(1,cha+1):
	        imdata = AIPSImage(data[0],data[1],data[2],i)
		imdata.zap()

    return(['CUBE','XIM',1,1])


def checkimdim(data,antfile):
    """
    """
    global workchannel
    from math import pi,sqrt
    from time import sleep
    #
    workchannel = 1
    chdata    = chanextract(data,1) # produce a sub-image
    uvdataset = simchanmerge(chdata,antfile,1/60.,1,-1)	 
    imdata    = AIPSImage(chdata[0],chdata[1],chdata[2],int(1))
    # input image info
    freqdim   = getimheader(chdata,gethead='FREQ')
    nchannels = freqdim[3]
    dfreq     = freqdim[2]
    obsfreqc  = freqdim[1]
    obsfreq   = freqdim[0]
    #
    raimage    = getimheader(data,gethead='RA---SIN')
    decimage   = getimheader(data,gethead='DEC--SIN')
    #
    testuv = AIPSUVData(uvdataset[0],uvdataset[1],uvdataset[2],uvdataset[3])
    antab  = testuv.table('AN',1)
    xyz = []
    for row in antab:
        xyz.append(row['stabxyz'])

    diss = []
    for i in range(len(xyz)):
        for j in range(len(xyz)):
            if xyz[i][0] != xyz[j][0] and xyz[i][1] != xyz[j][1] and xyz[i][2] != xyz[j][2]:
                diss.append(sqrt((xyz[i][0]-xyz[j][0])**2+(xyz[i][1]-xyz[j][1])**2+(xyz[i][2]-xyz[j][2])**2))
    dis = []
    for i in range(len(diss)):
        if diss[i] > 0:
            dis.append(diss[i])

    imdata.clrstat()
    testuv.zap()
    imdata.zap()

    maxuvdis = max(dis)
    minuvdis = min(dis)

    light_c = 299792458.0        # m s-1
    wavelengh = light_c/obsfreq

    # half power beam width = 1.02 * 
    # first null              1.22 *

    fwhm = 1.02*wavelengh/maxuvdis*360.0/(2.0*pi)

    factor = (decimage[2]/(fwhm/3.0))

    print '\nArray resolution is [arcsec]: ',(60*60*fwhm)
    print 'Array resolution Nyquist sampling [arcsec]: ',(60*60*fwhm)/3.0
    print 'Image resolution in [arcsec]: ',decimage[2]*60*60,'\n\n'
    sleep(2)

    if factor > 1:
        if int(factor)%2 == 1:
            factor = int(factor) + 1
        else:
            factor = int(factor) + 2

        from math import log,ceil

        npix = decimage[3]

        if (log(int(npix * factor) + 1)/log(2)) - int(log(int(npix * factor) + 1)/log(2)) > 0:
            newdim = 2**ceil(log(npix * factor)/log(2))
            factor = ceil(newdim/npix)

        
        print '\n\nIMAGE resolution is to low increase size by factor of: ',factor
        print '\nNew image dimension are', newdim,' pixels\n\n'

        imdata    = AIPSImage(data[0],data[1],data[2],int(1))

        blwup             = AIPSTask('blwup')
        blwup.indata      = imdata
        blwup.outname     = data[0]
        blwup.outclass    = 'LARGE'
        blwup.outseq      = int(data[2])
        blwup.outdisk     = int(data[3])
        blwup.factor      = factor
        blwup.msgkill     = noutput
        blwup.go()
        #
        imdata.zap()
        #
        return([data[0],'LARGE',int(blwup.outseq),int(blwup.outdisk)])
    else:
        return(data)

# --------- The actual program

import os
import sys
import re
#


if len(sys.argv) == 1:
        print '\n------- HRK Array Simulator -------\n'
        print 'use like:\n'
        print ' ParselTongue ',sys.argv[0],' AIPSNUMBER  IMAGEFILENAME UVCONFILENAME\n'
	print ' OPTIONAL: #CLEAN OBSERVATION[hrs] INTEGRATIONperVisibility[min] IMWEIGHTS[-5 pure uniform, 5 natural] IMAGESIZE\n'

	print 'possible input: ParselTongue SKADS_SIMULATOR.py 555 IMAGEFILE UVCONFILE 100 24 10'
        print '--------------------------------------------------------------------------------------------'
        print '\n\n\nPLEASE NOTE THAT THE IMPUT IMAGE DIMENSION NEED TO BE n**2  (e.g. 256, 512,... max 8096)\n' 
        print 'PLEASE NOTE THAT THE IMPUT IMAGE PIXEL DIMENSION NEEDS TO BE 1/3 of the maximum baseline resolution\n' 
        print '\n Enjoy !'
        if docleanup == 1:
            os.system('rm -fr '+cwd+'/da00')
            os.system('rm -fr '+cwd+'/aipsdisk')
        sys.exit(-1)
	
doask      =  1 
makecube   = -1
pmha       = 0.5
intpvismin = 10
cccomp     = 200
robust     = 0
samedim    = 1
imagesize  = -1

global cleanup
cleanup = 1

if len(sys.argv) == 4:

    # Input image file
    imagefile = sys.argv[2]
    if checkfi(imagefile) == -1:
	print 'File ',imagefile,' does not exist'
	sys.exit(-1)
   
    # Input antenna file
    antfile = sys.argv[3]

    if checkfi(antfile) == -1:
	print 'File ',antfile,' does not exist'
	sys.exit(-1)
 
elif len(sys.argv) == 5:

    # Input image file
    imagefile = sys.argv[2]
    if checkfi(imagefile) == -1:
	print 'File ',imagefile,' does not exist'
	sys.exit(-1)
   
    # Input antenna file
    antfile = sys.argv[3]

    if checkfi(antfile) == -1:
	print 'File ',antfile,' does not exist'
	sys.exit(-1)

    cccomp    = abs(int(sys.argv[4]))
    if int(sys.argv[4]) < 0:
        doask = -1


elif len(sys.argv) == 6:

    # Input image file
    imagefile = sys.argv[2]
    if checkfi(imagefile) == -1:
	print 'File ',imagefile,' does not exist'
	sys.exit(-1)
   
    # Input antenna file
    antfile = sys.argv[3]

    if checkfi(antfile) == -1:
	print 'File ',antfile,' does not exist'
	sys.exit(-1)

    cccomp    = abs(int(sys.argv[4]))
    if int(sys.argv[4]) < 0:
        doask = -1

    pmha    = float(sys.argv[5])/2.



elif len(sys.argv) == 7:

    # Input image file
    imagefile = sys.argv[2]
    if checkfi(imagefile) == -1:
	print 'File ',imagefile,' does not exist'
	sys.exit(-1)
   
    # Input antenna file
    antfile = sys.argv[3]

    if checkfi(antfile) == -1:
	print 'File ',antfile,' does not exist'
	sys.exit(-1)

    cccomp    = abs(int(sys.argv[4]))
    if int(sys.argv[4]) < 0:
        doask = -1

    pmha       = float(sys.argv[5])/2.

    intpvismin = float(sys.argv[6])

elif len(sys.argv) == 8:

    # Input image file
    imagefile = sys.argv[2]
    if checkfi(imagefile) == -1:
	print 'File ',imagefile,' does not exist'
	sys.exit(-1)
   
    # Input antenna file
    antfile = sys.argv[3]

    if checkfi(antfile) == -1:
	print 'File ',antfile,' does not exist'
	sys.exit(-1)

    cccomp    = abs(int(sys.argv[4]))
    if int(sys.argv[4]) < 0:
        doask = -1

    pmha       = float(sys.argv[5])/2.

    intpvismin = float(sys.argv[6])

    robust  = float(sys.argv[7])

elif len(sys.argv) == 9:

    # Input image file
    imagefile = sys.argv[2]
    if checkfi(imagefile) == -1:
	print 'File ',imagefile,' does not exist'
	sys.exit(-1)
   
    # Input antenna file
    antfile = sys.argv[3]

    if checkfi(antfile) == -1:
	print 'File ',antfile,' does not exist'
	sys.exit(-1)

    cccomp    = abs(int(sys.argv[4]))
    if int(sys.argv[4]) < 0:
        doask = -1

    pmha       = float(sys.argv[5])/2.

    intpvismin = float(sys.argv[6])

    robust  = float(sys.argv[7])

    samedim   = -1
    imagesize = int(sys.argv[8])

else:
    sys.exit(-1)


dosimulation = 1
if dosimulation > 0:
 
    # organizing stuff
    simoutname = 'SM_SKADSIM'
    #
    ###
    #
    # ---------------------------------------------------------
    try:
	checkfile = 1
	os.stat(antfile)
    except os.error:
	checkfile =-1
	pass
    if 0 > checkfile: 
	# produce new antennadistribution file
	print 'Produced first a new antenna file !!!',str(antfile)
	os.exit(-1)
    else:
	print 'Used antenna file: ',str(antfile)
    # ---

    from AIPS import AIPS
    from AIPSTask import AIPSTask, AIPSList
    from AIPSData import AIPSUVData, AIPSImage,AIPSCat
    from Wizardry.AIPSData import AIPSUVData as WizAIPSUVData
    #
    global workchannel
    #
    # PARSELTongue message setting 
    #
    outpname  = simoutname  # 'OUTPUTSIM'    # all outputfiles start with
    #
    indata  = 1                 # determine the AIPS DISK
    outdata = int(indata)

    noutput = -2
    #
    mergelog    = outpname+'.log'
    mergecheck  = outpname+'_CHECKoutput'
    #

    # INITIALISE LOG
    #
    AIPS.log      = open('./'+mergelog,'a')
    #

    #if len(sys.argv) == 2:
    AIPS.userno = int(sys.argv[1])
    #else:   
	    # 5000-5009 calibration
	    # 5010-5020 imaging
	    #
#	    AIPS.userno = 555


    print '\n\n AIPS USERID IS SET TO ', AIPS.userno,'\n\n'


    if len(imagefile) > 0:
	# Import the image file
	#
        cwd = os.getcwd()
        #
	fitld          = AIPSTask('fitld')
	fitld.outdisk  = int(outdata)
	fitld.outclass = 'IMAGIN'
	fitld.douvcomp = -1
	fitld.digicor  = -1
        #
        #fitld.datain   = '/tmp/'+imagefile
        #print '/tmp/'+imagefile
	fitld.datain  = 'PWD:'+imagefile
	fitld.outname = 'IMAG'
	fitld.msgkill = noutput
	fitld.go()

	# get some info from the original image
	data   = [fitld.outname,fitld.outclass,int(fitld.outdisk),1]
	inputdata = [fitld.outname,fitld.outclass,int(fitld.outdisk),1]
	freqdim  = getimheader(data,gethead='FREQ')
	nchannels = freqdim[3]
	dfreq     = freqdim[2]
	obsfreqc  = freqdim[1]
	obsfreq   = freqdim[0]
        #

        if getimheader(data,gethead='STOKES')[0] == 0:
            print 'hallo wrong stokes parametrer '
            putimheader(data,'STOKES','crval',1)
            putimheader(data,'STOKES','crpix',1)

        # input image info
        raimage    = getimheader(data,gethead='RA---SIN')
        decimage   = getimheader(data,gethead='DEC--SIN')
        
        xydimrach     = raimage  # getimheader(data,gethead='RA---SIN')
        xydim         = decimage # getimheader(data,gethead='DEC--SIN')
        #
        # Here images is check on power of 2 required for uvcon 
        #
        data = cropimage(data)

        # Here check if the image has the correct resolution
        # here UVCON runs the first time to get the correct reolution
        data      = checkimdim(data,antfile)
        inputdata = [data[0],data[1],data[2],data[3]]

    # Statistic
    # input image either the original one or the croped version
    inputstats    = getimstat(data)



    # determine the center of the image
    if int(decimage[1]) != int(decimage[3]/2.):
        decimcentre =  decimage[0] + (decimage[3]/2. - decimage[1] + 1) * decimage[2]
    else:
        decimcentre = decimage[0]


    if samedim == 1:
	cellsi  = [abs(abs(3600*decimage[2])),abs(abs(3600*decimage[2]))]
    else:
	cellsi  = [0,0]
    #    
    workchannel = 1
    if nchannels > 1:
        if doask == 1:
            mkcube = raw_input('\n Input file is a cube. Make multifreq synthesis instead of cube ? [y,n] ')
            if len(mkcube)!=0:
                if mkcube == 'n' or mkcube == 'N':
                    makecube = 1 
        else:
            makecube = 1


	if makecube < 0:
	    # Produce a single channel UVdataset taken into account spectral index from 
	    # various sources
	    chdata    = chanextract(data,1)
	    uvdataset = simchanmerge(chdata,antfile,pmha,intpvismin,-1)	         

	    # delete the input image
	    imdata = AIPSImage(chdata[0],chdata[1],chdata[2],chdata[3])
	    imdata.clrstat()
	    imdata.zap()

	    for i in range(2,nchannels+1):
		workchannel = i	    
		chdata      = chanextract(data,i)
		uvdataset   = simchanmerge(chdata,antfile,pmha,intpvismin,1,uvdataset)	         
		# delete the input image
		imdata = AIPSImage(chdata[0],chdata[1],chdata[2],chdata[3])
                if imdata.exists() == True:
                    imdata.clrstat()
                    imdata.zap()

	    finalimage = mkimage(uvdataset,data,cellsi,imagesize,cccomp,robust)[0]

	else:
	    fimag =	simchan(chanextract(data,1),antfile,pmha,intpvismin,cellsi,imagesize,cccomp,robust)	         
	    for i in range(2,nchannels+1):
		simchan(chanextract(data,i),antfile,pmha,intpvismin,fimag[1][0],imagesize,cccomp,robust)
		workchannel = i + 1		
	    finalimage = chanmerge(fimag[0],nchannels)
	    # delete the input cube
	    imdata = AIPSImage(inputdata[0],inputdata[1],inputdata[2],inputdata[3])
            if imdata.exists() == True:
                imdata.clrstat()
                imdata.zap()
    else:
	finalimage = simchan(data,antfile,pmha,intpvismin,cellsi,imagesize,cccomp,robust)	         
	finalimage = finalimage[0] 


    xydimfimage   = getimheader(finalimage,gethead='DEC--SIN')
    stats         = getimstat(finalimage)
    fifreqdim     = getimheader(finalimage,gethead='FREQ')
    finchannels   = fifreqdim[3]


    fname = 'SM_'+imagefile[:len(imagefile)-5] 
    fnamext = '.fits'
    file2image = '_CC'+str(cccomp)

    os.system('/bin/mv SM_UVPLT_CHA1.jpeg '+fname+'_UVPLT.jpeg')

    if checkfi(fname+file2image+fnamext) == 1:
	os.system('mv '+fname+file2image+fnamext+' '+fname+file2image+fnamext+'.org')


    save2disk(finalimage,fname,file2image,fnamext,doclean=1,noutput=-2)


    print '\n\n---- Simulator tool ----\n'

    print '- Simulation'
    print '  Observation lengh [hour] :                 ',2*pmha
    print '  Hour angle [hour] :                        ',pmha
    print '  Declination [deg] :                        ',decimcentre
    print '  integration per visibility [min]:          ',intpvismin
    print '  number of clean components :               ',cccomp
    print '  array layout used :                        ', antfile,'\n'

    print '- Images'
    inputype = 'image'
    if nchannels > 1:
	print '  input',inputype,' # channels :             ',nchannels
	inputype = 'cube'
    print '  input',inputype,':                          ',imagefile
    print '  input',inputype,'[pixel, arcsec/pixel] :    ',int(xydim[3]) ,', %.2e'%(xydim[2]*60*60)

    if nchannels > 1:
	print '  simulated',inputype,' # channels :          ',finchannels
    print '  simulated',inputype,':                      ',fname+file2image+fnamext
    print '  simulated',inputype,'[pixel, arcsec/pixel] :',int(xydimfimage[3]) ,', %.2e'%(xydimfimage[2]*60*60)


    print '\n- Statistics excluding 5% of the [input | output]'
    print '  rms imagr         [Jy] : [%e'%inputstats[0],' | %e'%stats[0],']'
    print '  avg imagr         [Jy] : [%e'%inputstats[1],' | %e'%stats[1],']'
    print '  maximum flux      [Jy] : [%e'%inputstats[3],' | %e'%stats[3],']'
    print '  minimum flux      [Jy] : [%e'%inputstats[2],' | %e'%stats[2],']'
    print '\n'


    print 'uvcoverage file produced : ',fname+'_UVPLT.jpeg'

    print '\n-- As AIPS this comes with ABSOLUTELY NO WARRANTY--'


if docleanup == 1:
    os.system('rm -fr '+cwd+'/da00')
    os.system('rm -fr '+cwd+'/aipsdisk')

